//
//  SCSMountainChartView.swift
//  SciChartSwiftDemo
//
//  Created by Mykola Hrybeniuk on 6/2/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import UIKit
import SciChart

class SCSMountainChartView: SCSBaseChartView {

    // MARK: Overrided Functions
    
    override func completeConfiguration() {
        super.completeConfiguration()
        addAxis()
        addModifiers()
        addDataSeries()
    }
    
    func addModifiers() {
        
        let xAxisDragmodifier = SCIXAxisDragModifier()
        xAxisDragmodifier.modifierName = xAxisDragModifierName
        xAxisDragmodifier.axisId = axisXId
        xAxisDragmodifier.dragMode = .Scale
        xAxisDragmodifier.clipModeX = .None
        
        let yAxisDragmodifier = SCIYAxisDragModifier()
        yAxisDragmodifier.modifierName = yAxisDragModifierName
        yAxisDragmodifier.axisId = axisYId
        yAxisDragmodifier.dragMode = .Pan
        
        let extendZoomModifier = SCIZoomExtentsModifier()
        extendZoomModifier.modifierName = extendZoomModifierName
        
        let pinchZoomModifier = SCIPinchZoomModifier()
        pinchZoomModifier.modifierName = pinchZoomModifierName
        
        let rolloverModifier = SCIRolloverModifier()
        rolloverModifier.modifierName = rolloverModifierName
//        rolloverModifier.style.tooltipSize = CGSizeMake(200, CGFloat.NaN)
        rolloverModifier.style.hitTestMode = .Vertical
        
        let groupModifier = SCIModifierGroup(childModifiers: [xAxisDragmodifier, yAxisDragmodifier, pinchZoomModifier, extendZoomModifier, rolloverModifier])
        
        chartSurface.chartModifier = groupModifier
    }
    
    // MARK: Private Functions
    
    private func addAxis() {
        let axisStyle = generateDefaultAxisStyle()
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultDateTimeAxis(withAxisStyle: axisStyle, axisId: axisXId), isXAxis: true)
        chartSurface.attachAxis(SCSFactoryAxis.createDefaultNumericAxis(withAxisStyle: axisStyle, axisId: axisYId), isXAxis: false)
    }
    
    private func addDataSeries () {
       
        
        let brush = SCIBrushLinearGradient(colorCodeStart: 0xff315897,
                                           finish: 0x88090E11,
                                           direction: .Vertical)
        let pen = SCIPenSolid(colorCode: 0xFF89a7d7, width: 0.5)
        
        
        chartSurface.attachRenderableSeries(getMountainRenderSeries(withBrush: brush, and: pen))
        chartSurface.invalidateElement()
    }
    
    
    private func getMountainRenderSeries(withBrush brush:SCIBrushLinearGradient, and pen: SCIPenSolid) -> SCIFastMountainRenderableSeries {
        
        let dataSeries = SCIXyDataSeries(XType: .DateTime, YType: .Float)
        dataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        DataManager.loadDataFromFile(dataSeries,
                                     fileName: "FinanceData",
                                     startIndex: 0,
                                     increment: 1,
                                     reverse: true)
        
        
        let mountainRenderSeries = SCIFastMountainRenderableSeries()
        mountainRenderSeries.style.areaBrush = brush
        mountainRenderSeries.style.borderPen = pen
        mountainRenderSeries.xAxisId = axisXId
        mountainRenderSeries.yAxisId = axisYId
        mountainRenderSeries.dataSeries = dataSeries
               
        return mountainRenderSeries
        
    }
    
    
    
}
