//
//  SCSRealtimeTickingStockChartView.swift
//  SciChartSwiftDemo
//
//  Created by Yaroslav Pelyukh on 7/19/16.
//  Copyright © 2016 SciChart Ltd. All rights reserved.
//

import Foundation
import SciChart

class SCSRealtimeTickingStockChartView: UIView {
    
    var sciChartView1 :SCIChartSurfaceView!
    var sciChartView2: SCIChartSurfaceView!
    var surface1: SCIChartSurface!
    var surface2: SCIChartSurface!
    
    var szpm: SCIMultiSurfaceModifier!
    var szem: SCIMultiSurfaceModifier!
    var spzm: SCIMultiSurfaceModifier!
    
    var ohlcDataSeries: SCIOhlcDataSeries!
    var avgDataSeries: SCIXyDataSeries!
    var mountainDataSeries: SCIXyDataSeries!
    
    var xAxis: SCIAxis2D!
    
    var renderableSeries: SCIRenderableSeriesBase!
    var avgRenderableSeries: SCIRenderableSeriesBase!
    var marketDataService: MarketDataService!
    var msa: SCDMovingAverage!
    var lastPrice: SCDMultiPaneItem!
    var box: SCIBoxAnnotation!
    
    var timer: NSTimer?
    var counter:Int32 = 0
    var seriesCount:Int32 = 0
    var countUpdater:Int32 = 0
    
    var lastMarker: SCIAxisMarkerAnnotation!
    var lastUpColor: UIColor!
    var lastDownColor: UIColor!
    var averageMarker: SCIAxisMarkerAnnotation!
    var drawZoomRectangle = false
    
    var alertView: UIAlertController!
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        weak var wSelf = self
        alertView = UIAlertController(title: "Series type", message: "Select series type for the top scichart surface", preferredStyle: .ActionSheet)
        var action = UIAlertAction(title: "CandlestickRenderableSeries", style: .Default, handler: {(action: UIAlertAction) -> Void in
            wSelf!.surface1.detachRenderableSeries(wSelf!.renderableSeries)
            wSelf!.updateToCandlestickRenderableSeries()
            wSelf!.updateRenderableSeriesType()
        })
        alertView.addAction(action)
        action = UIAlertAction(title: "OhlcRenderableSeries", style: .Default, handler: {(action: UIAlertAction) -> Void in
            wSelf!.surface1.detachRenderableSeries(wSelf!.renderableSeries)
            wSelf!.updateToOhlcRenderableSeries()
            wSelf!.updateRenderableSeriesType()
        })
        alertView.addAction(action)
        action = UIAlertAction(title: "MountainRenderableSeries", style: .Default, handler: {(action: UIAlertAction) -> Void in
            wSelf!.surface1.detachRenderableSeries(wSelf!.renderableSeries)
            wSelf!.updateToMountainRenderableSeries()
            wSelf!.updateRenderableSeriesType()
        })
        alertView.addAction(action)
        
        configureChartSurfaceViews()
        configureChartSurfaces()
        configureSeriesData()
        initializeTopSurfaceData()
        initializeBottomSurfaceData()
        continueTicking()
    }
    
    required init?(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)
    }
    
    override func willMoveToWindow(newWindow: UIWindow?) {
        super.willMoveToWindow(newWindow);
        
        if newWindow == nil{
            if timer != nil{
                timer!.invalidate()
                timer = nil
            }
        }
        else{
            if (timer == nil) {
            timer = NSTimer.scheduledTimerWithTimeInterval(0.1,
                                                           target: self,
                                                           selector: #selector(SCSRealtimeTickingStockChartView.updateData),
                                                           userInfo: nil,
                                                           repeats: true)}
        }
    }
    
    override func removeFromSuperview() {
        super.removeFromSuperview()
        if timer != nil {
            timer!.invalidate()
            timer = nil
        }
    }
    
    func configureSeriesData(){
        counter = 0
        seriesCount = 60
        countUpdater = seriesCount
        
        msa = SCDMovingAverage(length: 50)
        let initialDate = NSDate(timeIntervalSinceNow: 500)
        
        self.marketDataService = MarketDataService(startDate: initialDate, timeFrameMinutes: 500, tickTimerIntervals: 0)
        let prices = marketDataService.getHistoricalData(seriesCount)
        
        ohlcDataSeries = SCIOhlcDataSeries(XType: .DateTime, YType: .Double)
        ohlcDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        for i in 0..<prices.count {
            let item = prices[i]
            ohlcDataSeries.appendX(SCIGeneric(item.dateTime), open: SCIGeneric(item.open), high: SCIGeneric(item.high), low: SCIGeneric(item.low), close: SCIGeneric(item.close))
        }
        
        avgDataSeries = SCIXyDataSeries(XType: .DateTime, YType: .Float)
        avgDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        for i in 0..<prices.count {
            let item = prices[i]
            avgDataSeries.appendX(SCIGeneric(item.dateTime), y: SCIGeneric(msa.push((item.close)).current))
            self.lastPrice = item as! SCDMultiPaneItem
        }
    }
    
    func configureChartSurfaces() {
        //  Initializing SciChart Modifiers
        szpm = SCIMultiSurfaceModifier(modifierType: SCIZoomPanModifier.self)
        szem = SCIMultiSurfaceModifier(modifierType: SCIZoomExtentsModifier.self)
        spzm = SCIMultiSurfaceModifier(modifierType: SCIPinchZoomModifier.self)
        
        //  Initializing top scichart surface
        surface1 = SCIChartSurface(view: sciChartView1)
        surface1.style.backgroundBrush = SCIBrushSolid(colorCode: 0xFF1e1c1c)
        surface1.style.seriesBackgroundBrush = SCIBrushSolid(colorCode: 0xFF1e1c1c)
        
        //  Initializing bottom scichart surface
        surface2 = SCIChartSurface(view: sciChartView2)
        surface2.style.backgroundBrush = SCIBrushSolid(colorCode: 0xFF1e1c1c)
        surface2.style.seriesBackgroundBrush = SCIBrushSolid(colorCode: 0xFF1e1c1c)
        
        //  Initializing box annotation
        box = SCIBoxAnnotation()
        box.xAxisId = "X2"
        box.yAxisId = "Y2"
        box.coordMode = .RelativeY
        box.style.fillBrush = SCIBrushSolid(colorCode: 0x20FF7000)
        surface2.annotation = box
    }
    
    func continueTicking(){
        if(timer == nil){
            timer = NSTimer.scheduledTimerWithTimeInterval(0.1,
                                                           target: self,
                                                           selector: #selector(SCSRealtimeTickingStockChartView.updateData),
                                                           userInfo: nil,
                                                           repeats: true)}
    }
    
    func pauseTicking(){
        if timer == nil{
            return
        }
        timer!.invalidate()
        timer = nil
    }
    
    func updateToOhlcRenderableSeries() {
        renderableSeries = SCIFastOhlcRenderableSeries()
        renderableSeries.xAxisId = "X1"
        renderableSeries.yAxisId = "Y1"
        renderableSeries.dataSeries = ohlcDataSeries
        (renderableSeries as! SCIFastOhlcRenderableSeries).style.upWickPen = SCIPenSolid(colorCode: 0xFF4440a6, width: 1.0)
        (renderableSeries as! SCIFastOhlcRenderableSeries).style.downWickPen = SCIPenSolid(colorCode: 0xFF3ba13d, width: 1.0)
    }
    
    func updateToCandlestickRenderableSeries() {
        renderableSeries = SCIFastCandlestickRenderableSeries()
        renderableSeries.xAxisId = "X1"
        renderableSeries.yAxisId = "Y1"
        renderableSeries.dataSeries = ohlcDataSeries
        (renderableSeries as! SCIFastCandlestickRenderableSeries).style.drawBorders = false
        (renderableSeries as! SCIFastCandlestickRenderableSeries).style.upWickPen = SCIPenSolid(colorCode: 0xFF4440a6, width: 1.0)
        (renderableSeries as! SCIFastCandlestickRenderableSeries).style.downWickPen = SCIPenSolid(colorCode: 0xFF3ba13d, width: 1.0)
        (renderableSeries as! SCIFastCandlestickRenderableSeries).style.upBodyBrush = SCIBrushSolid(colorCode: 0xFF4440a6)
        (renderableSeries as! SCIFastCandlestickRenderableSeries).style.downBodyBrush = SCIBrushSolid(colorCode: 0xFF3ba13d)
    }
    
    func updateToMountainRenderableSeries() {
        renderableSeries = SCIFastMountainRenderableSeries()
        let brush = SCIBrushLinearGradient(colorCodeStart: 0x888f663a, finish: 0xff4f3820, direction: .Vertical)
        let pen = SCIPenSolid(colorCode: 0xffffe6c6, width: 0.5)
        (renderableSeries as! SCIFastMountainRenderableSeries).style.areaBrush = brush
        (renderableSeries as! SCIFastMountainRenderableSeries).style.borderPen = pen
        renderableSeries.xAxisId = "X1"
        renderableSeries.yAxisId = "Y1"
        renderableSeries.dataSeries = ohlcDataSeries
    }
    
    func changeSeriesType(sender: UIButton) {
        alertView.popoverPresentationController?.sourceRect = sender.bounds
        alertView.popoverPresentationController?.sourceView = sender
        let currentTopVC = currentTopViewController()
        currentTopVC.presentViewController(alertView, animated: true, completion: nil)
    }
    
    func currentTopViewController() -> UIViewController {
        var topVC = UIApplication.sharedApplication().delegate!.window!!.rootViewController!
        while topVC.isBeingPresented() {
            topVC = topVC.presentedViewController!
        }
        return topVC
    }
    
    func updateRenderableSeriesType() {
        surface1.detachRenderableSeries(avgRenderableSeries)
        surface1.attachRenderableSeries(renderableSeries)
        surface1.attachRenderableSeries(avgRenderableSeries)
        surface1.invalidateElement()
    }
    
    
    // MARK: Private Functions
    
    func configureChartSurfaceViews() {
        
        //  Initializing top scichart view
        sciChartView1 = SCIChartSurfaceView()
        sciChartView1.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(sciChartView1)
        
        //  Initializing bottom scichart view
        sciChartView2 =  SCIChartSurfaceView()
        sciChartView2.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(sciChartView2)
        
        weak var wSelf = self
        let panel = (NSBundle.mainBundle().loadNibNamed("SCSRealTimeTickingStocksControlPanel", owner: self, options: nil)!.first! as! SCSRealTimeTickingStocksControlPanelView)
        panel.translatesAutoresizingMaskIntoConstraints = false
        //Subscribing to the control view events
        panel.continueTickingTouched = {(sender: UIButton) -> Void in
            wSelf!.continueTicking()
        }
        panel.pauseTickingTouched = {(sender: UIButton) -> Void in
            wSelf!.pauseTicking()
        }
        panel.seriesTypeTouched = {(sender: UIButton) -> Void in
            wSelf!.changeSeriesType(sender)
        }
        self.addSubview(panel)
        
        sciChartView1.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(sciChartView1)
        
        sciChartView2.translatesAutoresizingMaskIntoConstraints = false
        self.addSubview(sciChartView2)
        
        let layoutDictionary = ["SciChart1" : sciChartView1, "SciChart2" : sciChartView2, "Panel" : panel]
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[SciChart1]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[SciChart2]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("|-(0)-[Panel]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
        
        self.addConstraints(NSLayoutConstraint.constraintsWithVisualFormat("V:|-(0)-[Panel(40)]-(0)-[SciChart1]-(5)-[SciChart2(100)]-(0)-|",
            options: .DirectionLeadingToTrailing,
            metrics: nil,
            views: layoutDictionary))
    }
    
    func getOhlcRenderableSeries(isRevered: Bool, upBodyBrush upBodyColor: SCIBrushSolid, downBodyBrush downBodyColor: SCIBrushSolid, count: Int) -> SCIFastCandlestickRenderableSeries {
        renderableSeries = SCIFastCandlestickRenderableSeries()
        renderableSeries.xAxisId = "X1"
        renderableSeries.yAxisId = "Y1"
        renderableSeries.dataSeries = ohlcDataSeries
        (renderableSeries as! SCIFastCandlestickRenderableSeries).style.upWickPen = SCIPenSolid(colorCode: 0xFF4440a6, width: 0.7)
        (renderableSeries as! SCIFastCandlestickRenderableSeries).style.downWickPen = SCIPenSolid(colorCode: 0xFF3ba13d, width: 0.7)
        (renderableSeries as! SCIFastCandlestickRenderableSeries).style.upBodyBrush = SCIBrushSolid(colorCode: 0xFF4440a6)
        (renderableSeries as! SCIFastCandlestickRenderableSeries).style.downBodyBrush = SCIBrushSolid(colorCode: 0xFF3ba13d)
        return (renderableSeries as! SCIFastCandlestickRenderableSeries)
    }
    
    func getMountainRenderSeries(withBrush brush:SCIBrushLinearGradient, and pen: SCIPenSolid) -> SCIFastMountainRenderableSeries {
        
        mountainDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        
        for i in 0..<seriesCount {
            mountainDataSeries.appendX(SCIGeneric(i), y: ohlcDataSeries.highValues().valueAt(Int(i)))
        }
        
        let mountainRenderSeries = SCIFastMountainRenderableSeries()
        mountainRenderSeries.style.areaBrush = brush
        mountainRenderSeries.style.borderPen = pen
        mountainRenderSeries.xAxisId = "X2"
        mountainRenderSeries.yAxisId = "Y2"
        mountainRenderSeries.dataSeries = mountainDataSeries
        
        return mountainRenderSeries
        
    }
    
    func getAverageLine() -> SCIFastLineRenderableSeries {
        avgRenderableSeries = SCIFastLineRenderableSeries()
        (avgRenderableSeries as! SCIFastLineRenderableSeries).style.linePen = SCIPenSolid(colorCode: 0xFF00a5ff, width: 1.0)
        (avgRenderableSeries as! SCIFastLineRenderableSeries).xAxisId = "X1"
        (avgRenderableSeries as! SCIFastLineRenderableSeries).yAxisId = "Y1"
        (avgRenderableSeries as! SCIFastLineRenderableSeries).dataSeries = avgDataSeries
        return (avgRenderableSeries as! SCIFastLineRenderableSeries)
    }
    
    func getMountainRenderableSeries(areaBrush: SCIBrush2D, borderPen: SCIPen2D) -> SCIFastMountainRenderableSeries {
        mountainDataSeries = SCIXyDataSeries(XType: .DateTime, YType: .Float)
        mountainDataSeries.dataDistributionCalculator = SCIUserDefinedDistributionCalculator()
        for i in 0..<ohlcDataSeries.xValues().count() {
            mountainDataSeries.appendX(ohlcDataSeries.xValues().valueAt(Int(i)), y: ohlcDataSeries.highValues().valueAt(Int(i)))
        }
        
        let mountainRenderableSeries = SCIFastMountainRenderableSeries()
        mountainRenderableSeries.style.areaBrush = areaBrush
        mountainRenderableSeries.style.borderPen = borderPen
        mountainRenderableSeries.xAxisId = "X2"
        mountainRenderableSeries.yAxisId = "Y2"
        mountainRenderableSeries.dataSeries = mountainDataSeries
        return mountainRenderableSeries
    }
    
    func drawBoxAnnotation() {
        box.isEnabled = false
        box.xStart = xAxis!.visibleRange.min
        box.xEnd = xAxis!.visibleRange.max
        box.yStart = SCIGeneric(0)
        box.yEnd = SCIGeneric(1)
    }
    
    func updateData(timer: NSTimer) {
        let price = marketDataService.getNextBar()
        if lastPrice != nil && lastPrice.dateTime == price.dateTime {
            // TODO: compare dates in proper way
            ohlcDataSeries.updateAt(ohlcDataSeries.count() - 1, open: SCIGeneric(price.open), high: SCIGeneric(price.high), low: SCIGeneric(price.low), close: SCIGeneric(price.close))
            msa.update(price.close)
            avgDataSeries.updateAt(Int32(ohlcDataSeries.xValues().count()) - 1, XValue: SCIGeneric(price.dateTime), YValue: SCIGeneric(msa.current))
            mountainDataSeries.updateAt(Int32(mountainDataSeries.xValues().count()) - 1, XValue: SCIGeneric(price.dateTime), YValue: SCIGeneric(price.high))
        }
        else{
            ohlcDataSeries.appendX(SCIGeneric(price.dateTime), open: SCIGeneric(price.open), high: SCIGeneric(price.high), low: SCIGeneric(price.low), close: SCIGeneric(price.close))
            avgDataSeries.appendX(SCIGeneric(price.dateTime), y: SCIGeneric(msa.push(price.close).current))
            mountainDataSeries.appendX(SCIGeneric(price.dateTime), y: SCIGeneric(price.high))
            let visibleRangeMax: Double = SCIGenericDate(xAxis!.visibleRange.max).timeIntervalSince1970
            let lastItem: Double = SCIGenericDate(ohlcDataSeries.xValues().valueAt(ohlcDataSeries.xValues().count() - 1)).timeIntervalSince1970
            if visibleRangeMax >= lastItem {
                let priorItem: Double = SCIGenericDate(ohlcDataSeries.xValues().valueAt( ohlcDataSeries.xValues().count() - 2)).timeIntervalSince1970
                let visibleRangeMin: Double = SCIGenericDate(xAxis!.visibleRange.min).timeIntervalSince1970
                let leftItem: Double = SCIGenericDate(ohlcDataSeries.xValues().valueAt(0)).timeIntervalSince1970
                var min: Double
                if visibleRangeMin >= leftItem {
                    min = visibleRangeMin + (lastItem - priorItem)
                }
                else {
                    min = visibleRangeMin + (lastItem - priorItem)
                }
                xAxis!.visibleRange = SCIDateRange(min: SCIGeneric(min), max: SCIGeneric(lastItem + (visibleRangeMax - priorItem)))
            }
        }
        
        self.lastMarker.position = SCIGeneric(price.close)
        if price.close > price.open {
            self.lastMarker.style.backgroundColor = lastUpColor
        }
        else {
            self.lastMarker.style.backgroundColor = lastDownColor
        }
        self.averageMarker.position = SCIGeneric(msa.current)
        self.lastPrice = price
        surface1.invalidateElement()
        drawBoxAnnotation()
        surface2.invalidateElement()
        
    }
    
    func initializeTopSurfaceData() {
        let majorPen = SCIPenSolid(colorCode: 0xFF393532, width: 0.5)
        let gridBandPen = SCIBrushSolid(colorCode: 0xE1232120)
        let minorPen = SCIPenSolid(colorCode: 0xFF262423, width: 0.5)
        let textFormatting = SCITextFormattingStyle()
        textFormatting.fontSize = 16
        textFormatting.fontName = "Helvetica"
        textFormatting.colorCode = 0xFFafb3b6
        
        //Initializing axes and attaching them to the surface
        let axisStyle = SCIAxisStyle()
        axisStyle.majorTickBrush = majorPen
        axisStyle.gridBandBrush = gridBandPen
        axisStyle.majorGridLineBrush = majorPen
        axisStyle.minorTickBrush = minorPen
        axisStyle.minorGridLineBrush = minorPen
        axisStyle.labelStyle = textFormatting
        axisStyle.drawMinorGridLines = true
        axisStyle.drawMajorBands = true
        let axis = SCINumericAxis()
        axis.style = axisStyle
        axis.axisId = "Y1"
        axis.cursorTextFormatting = "%.02f"
        axis.growBy = SCIDoubleRange(min: SCIGeneric(0.1), max: SCIGeneric(0.1))
        axis.autoRange = .Always
        surface1.attachAxis(axis, isXAxis: false)
        
        xAxis = SCIDateTimeAxis()
        xAxis.axisId = "X1"
        let priorItem: Double = (SCIGenericDate(ohlcDataSeries.xValues().valueAt(ohlcDataSeries.xValues().count() - 2))).timeIntervalSince1970
        let lastItem: Double = SCIGenericDate(ohlcDataSeries.xValues().valueAt(ohlcDataSeries.xValues().count() - 1)).timeIntervalSince1970
        xAxis.visibleRange = SCIDateRange(min: ohlcDataSeries.xValues().valueAt(0), max: SCIGeneric(lastItem + (lastItem - priorItem)))
        (xAxis as! SCIDateTimeAxis).textFormatting = "dd/MM/yyyy"
        xAxis.style = axisStyle
        xAxis.growBy = SCIDoubleRange(min: SCIGeneric(0.0), max: SCIGeneric(0.1))
        surface1.attachAxis(xAxis, isXAxis: true)
        
        //Creating SciChart modifiers
        let x1Pinch = SCIAxisPinchZoomModifier()
        x1Pinch.axisId = "X1"
        x1Pinch.modifierName = "X1 Axis Pinch Modifier"
        let x1Drag = SCIXAxisDragModifier()
        x1Drag.axisId = "X1"
        x1Drag.dragMode = .Scale
        x1Drag.clipModeX = .None
        x1Drag.modifierName = "X1 Axis Drag Modifier"
        let y1Pinch = SCIAxisPinchZoomModifier()
        y1Pinch.axisId = "Y1"
        y1Pinch.modifierName = "Y1 Axis Pinch Modifier"
        let y1Drag = SCIYAxisDragModifier()
        y1Drag.axisId = "Y1"
        y1Drag.dragMode = .Pan
        y1Drag.modifierName = "Y1 Axis Drag Modifier"
        let pzm = SCIPinchZoomModifier()
        pzm.modifierName = "PinchZoom Modifier"
        let zem = SCIZoomExtentsModifier()
        zem.modifierName = "ZoomExtents Modifier"
        let zpm = SCIZoomPanModifier()
        zpm.modifierName = "PanZoom Modifier"
        
        //Initializing modifiers group and attaching it to the scichart surface
        let zommPanModifier = szpm.modifierForSurface(surface1)
        (zommPanModifier as! SCIZoomPanModifier).xyDirection = .XDirection
        let gm = SCIModifierGroup(childModifiers: [x1Pinch, y1Pinch, x1Drag, y1Drag, spzm, szem, szpm])
        self.surface1.chartModifier = gm
        
        surface1.attachRenderableSeries(getOhlcRenderableSeries(false, upBodyBrush: SCIBrushSolid(colorCode: 0xFF0f9cff), downBodyBrush: SCIBrushSolid(colorCode: 0xFF66ffff), count: Int(seriesCount)))
        surface1.attachRenderableSeries(getAverageLine())
        
        self.lastUpColor = UIColor.fromColorCode(0xFF4440a6)
        self.lastDownColor = UIColor.fromColorCode(0xFF3ba13d)
        self.lastMarker = SCIAxisMarkerAnnotation()
        self.lastMarker.yAxisId = "Y1"
        self.lastMarker.coordMode = .Absolute
        self.lastMarker.style.backgroundColor = lastUpColor
        
        self.averageMarker = SCIAxisMarkerAnnotation()
        self.averageMarker.yAxisId = "Y1"
        self.averageMarker.coordMode = .Absolute
        self.averageMarker.style.backgroundColor = UIColor.fromColorCode(0xFF00a5ff)
        self.surface1.annotation = SCIAnnotationGroup(childAnnotations: [averageMarker, lastMarker])
        surface1.invalidateElement()
    }
    
    func initializeBottomSurfaceData() {
        //Creating axes and attaching them to the scichart surface
        var axis = SCIDateTimeAxis()
        axis.axisId = "Y2"
        axis.style.drawMajorGridLines = false
        axis.isVisible = false
        axis.autoRange = .Always
        axis.growBy = SCIDoubleRange(min: SCIGeneric(0.1), max: SCIGeneric(0.1))
        surface2.attachAxis(axis, isXAxis: false)
        axis = SCIDateTimeAxis()
        axis.axisId = "X2"
        axis.autoRange = .Always
        axis.textFormatting = "dd/MM/yyyy"
        axis.style.drawMajorGridLines = false
        axis.isVisible = false
        axis.growBy = SCIDoubleRange(min: SCIGeneric(0.0), max: SCIGeneric(0.1))
        surface2.attachAxis(axis, isXAxis: true)
        
        //Creating modifiers and attaching them to the scichart surface
        let x2Pinch = SCIAxisPinchZoomModifier()
        x2Pinch.axisId = "X2"
        x2Pinch.modifierName = "Y2 Axis Pinch Modifier"
        
        let x2Drag = SCIXAxisDragModifier()
        x2Drag.axisId = "X2"
        x2Drag.dragMode = .Scale
        x2Drag.clipModeX = .None
        x2Drag.modifierName = "Y2 Axis Drag Modifier"
        let y2Pinch = SCIAxisPinchZoomModifier()
        y2Pinch.axisId = "Y2"
        y2Pinch.modifierName = "X2 Axis Pinch Modifier"
        let y2Drag = SCIYAxisDragModifier()
        y2Drag.axisId = "Y2"
        y2Drag.dragMode = .Pan
        y2Drag.modifierName = "X2 Axis Drag Modifier"
        let pzm = SCIPinchZoomModifier()
        pzm.modifierName = "PinchZoom Modifier"
        let zem = SCIZoomExtentsModifier()
        zem.modifierName = "ZoomExtents Modifier"
        let zpm = SCIZoomPanModifier()
        zpm.modifierName = "PanZoom Modifier"
        
        //Initializing modifiers group here and attaching to the surface
        let zoomPanModifier = szpm.modifierForSurface(surface2)
        (zoomPanModifier as! SCIZoomPanModifier).xyDirection = .XDirection
        let pinchZoomModifier = spzm.modifierForSurface(surface2)
        (pinchZoomModifier as! SCIPinchZoomModifier).xyDirection = .XDirection
        let gm = SCIModifierGroup(childModifiers: [szpm, spzm])
        self.surface2.chartModifier = gm
        let brush = SCIBrushLinearGradient(colorCodeStart: 0x888f663a, finish: 0xff4f3820, direction: .Vertical)
        let pen = SCIPenSolid(colorCode: 0xff8f663a, width: 0.5)
        //Attaching Renderable Series
        surface2.attachRenderableSeries(getMountainRenderableSeries(brush, borderPen: pen))
        surface2.invalidateElement()
    }
}





